<?php

namespace MiniLog;

class Logger
{

    const OPT_LEVEL = 'level';
    const LEVEL_SHORT = 'short';
    const LEVEL_FULL = 'full';
    const LEVEL_ALIGNED = 'aligned';
    
    const OPT_PRINTF = 'printf';
    
    const OPT_COLOR = 'color';
    const COLOR_NONE = 'none';
    const COLOR_AUTO = 'auto';
    const COLOR_FORCE = 'force';

    const OPT_FORMAT = 'format';
    const FORMAT_SHORT = '[{level}] {message}';
    const FORMAT_LONG = '{time} [{level}] {message}';
    const FORMAT_DEFAULT = '{time} {level} | {message}';
    
    const OPT_TIMESTAMP = 'timestamp';

    protected $styles = [
        'debug' => [ '32',  '32'    ],
        'info'  => [ '32',  '32;1'  ],
        'notice'=> [ '33',  '33'    ],
        'warn'  => [ '33',  '33;1'  ],
        'error' => [ '31',  '31;1'  ],
    ];

    public function __construct(array $options=[])
    {
        $defaults = [
            Logger::OPT_LEVEL   => Logger::LEVEL_SHORT,
            Logger::OPT_PRINTF  => true,
            Logger::OPT_COLOR   => Logger::COLOR_AUTO,
            Logger::OPT_FORMAT  => Logger::FORMAT_SHORT,
            Logger::OPT_TIMESTAMP => "d M H:i:s",
        ];
        $this->options = array_merge($defaults, $options);
    }
    
    public function getOption($key)
    {
        if (!array_key_exist($key,$this->options))
            return null;
        $this->options[$key] = $value;
    }
    
    public function setOption($key,$value)
    {
        if (!array_key_exist($key,$this->options))
            return false;
        $this->options[$key] = $value;
    }

    public function log($level, $fmt, ...$args)
    {
        // Use ansi if we are writing to a TTY
        $is_tty = defined("STDOUT")?(posix_isatty(STDOUT)):false;

        $level_str = $level;
        if ($this->options[self::OPT_LEVEL] == self::LEVEL_SHORT) {
            $level_str = $level_str[0];
        } elseif ($this->options[self::OPT_LEVEL] == self::LEVEL_ALIGNED) {
            $level_str = sprintf("%6s",$level_str);
        }
        $v = [
            'message' => sprintf($fmt,...$args),
            'level' => $level_str,
            'time' => date($this->options[self::OPT_TIMESTAMP]),
        ];

        $fmt = $this->options[self::OPT_FORMAT];
        if ($is_tty) {
            $v['message'] = "\e[".$this->styles[$level][1]."m".$v['message'];
            $fmt = "\e[".$this->styles[$level][0]."m".$fmt."\e[0m";
        }
                
        $out = preg_replace_callback('/\{(time|level|message)\}/', function ($match) use ($v) {
            return $v[$match[1]];
        }, $fmt);

        fwrite(STDOUT, $out."\n");
        //fprintf(STDOUT, "{$e1}[{$level_short}] {$e2}{$fmt}{$e0}\n", ...$args);
    }

    public function debug($fmt, ...$args)
    {
        $this->log('debug', $fmt, ...$args);
    }

    public function info($fmt, ...$args)
    {
        $this->log('info', $fmt, ...$args);
    }

    public function notice($fmt, ...$args)
    {
        $this->log('notice', $fmt, ...$args);
    }

    public function warn($fmt, ...$args)
    {
        $this->log('warn', $fmt, ...$args);
    }

    public function error($fmt, ...$args)
    {
        $this->log('error', $fmt, ...$args);
    }
}
